package de.fhdw.wtf.common.ast.type.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.Vector;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.AttributeModifierPrior;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.OperationModifier;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.BaseType;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassModifierAbstract;
import de.fhdw.wtf.common.ast.type.ClassModifierService;
import de.fhdw.wtf.common.ast.type.ClassModifierTransient;
import de.fhdw.wtf.common.ast.type.ClassModifierVisitable;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.AbstractToken;
import de.fhdw.wtf.common.token.keywords.PriorToken;
import de.fhdw.wtf.common.token.keywords.ServiceToken;
import de.fhdw.wtf.common.token.keywords.TransientToken;
import de.fhdw.wtf.common.token.keywords.VisitableToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;

/**
 * Tests some {@link de.fhdw.wtf.common.ast.type.Type}s.
 */
public class TypeTest {
	
	/**
	 * Tests an abstract class.
	 */
	@Test
	public void testClassAbstract() {
		final IdentifierToken className = IdentifierToken.create("className", Position.createDummyPosition());
		final Name name = UnqualifiedName.create(className);
		final Collection<ClassModifier> modifiers = new Vector<>();
		final Collection<Attribute> attributes = new Vector<>();
		final Collection<Type> superTypes = new Vector<>();
		final Collection<Operation> operations = new Vector<>();
		final Collection<Constructor> superConstructors = new Vector<>();
		final Collection<ClassType> subTypes = new Vector<>();
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						modifiers,
						attributes,
						superTypes,
						operations,
						superConstructors,
						className,
						subTypes);
		
		final AbstractToken abstractToken = AbstractToken.create(Position.createDummyPosition());
		modifiers.add(ClassModifierAbstract.create(abstractToken));
		
		assertTrue(clazz.isAbstract());
		assertFalse(clazz.isService());
		assertFalse(clazz.isTransient());
		assertFalse(clazz.isVisitable());
		// TODO -> Use visitor instead
		assertTrue(clazz instanceof RegularClassType);
	}
	
	/**
	 * Tests an service class.
	 */
	@Test
	public void testClassService() {
		final IdentifierToken className = IdentifierToken.create("className", Position.createDummyPosition());
		final Name name = UnqualifiedName.create(className);
		final Collection<ClassModifier> modifiers = new Vector<>();
		final Collection<Attribute> attributes = new Vector<>();
		final Collection<Type> superTypes = new Vector<>();
		final Collection<Operation> operations = new Vector<>();
		final Collection<Constructor> superConstructors = new Vector<>();
		final Collection<ClassType> subTypes = new Vector<>();
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						modifiers,
						attributes,
						superTypes,
						operations,
						superConstructors,
						className,
						subTypes);
		
		final ServiceToken serviceToken = ServiceToken.create(Position.createDummyPosition());
		modifiers.add(ClassModifierService.create(serviceToken));
		
		assertFalse(clazz.isAbstract());
		assertTrue(clazz.isService());
		assertFalse(clazz.isTransient());
		assertFalse(clazz.isVisitable());
		// TODO -> Use visitor instead
		assertTrue(clazz instanceof RegularClassType);
	}
	
	/**
	 * Tests an transient class.
	 */
	@Test
	public void testClassTransient() {
		final IdentifierToken className = IdentifierToken.create("className", Position.createDummyPosition());
		final Name name = UnqualifiedName.create(className);
		final Collection<ClassModifier> modifiers = new Vector<>();
		final Collection<Attribute> attributes = new Vector<>();
		final Collection<Type> superTypes = new Vector<>();
		final Collection<Operation> operations = new Vector<>();
		final Collection<Constructor> superConstructors = new Vector<>();
		final Collection<ClassType> subTypes = new Vector<>();
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						modifiers,
						attributes,
						superTypes,
						operations,
						superConstructors,
						className,
						subTypes);
		
		final TransientToken transientToken = TransientToken.create(Position.createDummyPosition());
		modifiers.add(ClassModifierTransient.create(transientToken));
		
		assertFalse(clazz.isAbstract());
		assertFalse(clazz.isService());
		assertTrue(clazz.isTransient());
		assertFalse(clazz.isVisitable());
		// TODO -> Use visitor instead
		assertTrue(clazz instanceof RegularClassType);
	}
	
	/**
	 * Tests an visitable class.
	 */
	@Test
	public void testClassVisitable() {
		final IdentifierToken className = IdentifierToken.create("className", Position.createDummyPosition());
		final Name name = UnqualifiedName.create(className);
		final Collection<ClassModifier> modifiers = new Vector<>();
		final Collection<Attribute> attributes = new Vector<>();
		final Collection<Type> superTypes = new Vector<>();
		final Collection<Operation> operations = new Vector<>();
		final Collection<Constructor> superConstructors = new Vector<>();
		final Collection<ClassType> subTypes = new Vector<>();
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						modifiers,
						attributes,
						superTypes,
						operations,
						superConstructors,
						className,
						subTypes);
		
		final VisitableToken visitableToken = VisitableToken.create(Position.createDummyPosition());
		modifiers.add(ClassModifierVisitable.create(visitableToken));
		
		assertFalse(clazz.isAbstract());
		assertFalse(clazz.isService());
		assertFalse(clazz.isTransient());
		assertTrue(clazz.isVisitable());
		// TODO -> Use visitor instead
		assertTrue(clazz instanceof RegularClassType);
	}
	
	/**
	 * Tests an exception.
	 */
	@Test
	public void testClassIsException() {
		final IdentifierToken className = IdentifierToken.create("className", Position.createDummyPosition());
		final Name name = UnqualifiedName.create(className);
		final Collection<ClassModifier> modifiers = new Vector<>();
		final Collection<Attribute> attributes = new Vector<>();
		final Collection<Type> superTypes = new Vector<>();
		final Collection<Operation> operations = new Vector<>();
		final Collection<Constructor> superConstructors = new Vector<>();
		final Collection<ClassType> subTypes = new Vector<>();
		
		final ClassType clazz =
				RegularClassType.create(
						name,
						modifiers,
						attributes,
						superTypes,
						operations,
						superConstructors,
						className,
						subTypes);
		
		assertFalse(clazz.isAbstract());
		assertFalse(clazz.isService());
		assertFalse(clazz.isTransient());
		// TODO -> Use visitor instead
		assertTrue(clazz instanceof RegularClassType);
	}
	
	/**
	 * Tests class toString()-Method.
	 */
	@Test
	public void testClassTypeToString() {
		final IdentifierToken classNameToken = IdentifierToken.create("className", Position.createDummyPosition());
		final Name className = UnqualifiedName.create(classNameToken);
		final Collection<ClassModifier> classModifiers = new Vector<>();
		final Collection<Attribute> attributes = new Vector<>();
		final Collection<Type> superTypes = new Vector<>();
		final Collection<Operation> operations = new Vector<>();
		final Collection<Constructor> superConstructors = new Vector<>();
		final Collection<ClassType> subTypes = new Vector<>();
		
		final ClassType clazz =
				RegularClassType.create(
						className,
						classModifiers,
						attributes,
						superTypes,
						operations,
						superConstructors,
						classNameToken,
						subTypes);
		
		final AbstractToken abstractToken = AbstractToken.create(Position.createDummyPosition());
		classModifiers.add(ClassModifierAbstract.create(abstractToken));
		
		final IdentifierToken typeNameToken = IdentifierToken.create("String", Position.createDummyPosition());
		final Name typeName = UnqualifiedName.create(typeNameToken);
		final BaseType bt = BaseType.create(typeName, typeNameToken, typeNameToken);
		
		final Collection<AttributeModifier> attributeModifiers = new Vector<>();
		final PriorToken priorToken = PriorToken.create(Position.createDummyPosition());
		attributeModifiers.add(AttributeModifierPrior.create(priorToken));
		final IdentifierToken attributeToken = IdentifierToken.create("Attribute", Position.createDummyPosition());
		final Attribute attribute = Attribute.create("Attribute", bt, attributeModifiers, attributeToken);
		attributes.add(attribute);
		
		final IdentifierToken superClassNameToken =
				IdentifierToken.create("superClass", Position.createDummyPosition());
		final Name superClassName = UnqualifiedName.create(superClassNameToken);
		final ByNameState state = ByNameState.create(superClassName);
		superTypes.add(TypeProxy.create(superClassNameToken, state));
		
		final ProductType pT = ProductType.create(BracketOpenToken.create(Position.createDummyPosition()));
		final SumType sT = SumType.create(CurlyBracketOpenToken.create(Position.createDummyPosition()));
		final IdentifierToken operationToken = IdentifierToken.create("Operation", Position.createDummyPosition());
		final Operation operation =
				Operation.create("Operation", new Vector<OperationModifier>(), pT, clazz, sT, operationToken);
		operations.add(operation);
		
		final String expected =
				"className:class=superClass + { \n" + "Attribute:String prior; \n"
						+ "Operation:[[()->{}]]; \n} abstract;";
		assertEquals(expected, clazz.toString());
	}
	
	/**
	 * Tests toString() of ListType.
	 */
	@Test
	public void testListTypeToString() {
		final IdentifierToken string = IdentifierToken.create("String", Position.createDummyPosition());
		final Name typeName = UnqualifiedName.create(string);
		final BaseType stringType = BaseType.create(typeName, string, string);
		final ListType listType = ListType.create(string, stringType);
		final String expected = "String*";
		
		assertEquals(expected, listType.toString());
	}
	
	/**
	 * Tests toString() of ListType with MapType.
	 */
	@Test
	public void testListTypeMapTypeToString() {
		final IdentifierToken string = IdentifierToken.create("String", Position.createDummyPosition());
		final IdentifierToken integer = IdentifierToken.create("Integer", Position.createDummyPosition());
		final Name typeName = UnqualifiedName.create(string);
		final BaseType stringType = BaseType.create(typeName, string, string);
		
		final Name typeNameInteger = UnqualifiedName.create(integer);
		final BaseType integerType = BaseType.create(typeNameInteger, integer, integer);
		final SquareBracketOpenToken sbo = SquareBracketOpenToken.create(Position.createDummyPosition());
		final MapType mapType = MapType.create(sbo, stringType, integerType);
		final ListType listType = ListType.create(sbo, mapType);
		
		final String expected = "[String->Integer]*";
		assertEquals(expected, listType.toString());
	}
	
	/**
	 * Tests toString() of MapType.
	 */
	@Test
	public void testMapTypeToString() {
		final IdentifierToken string = IdentifierToken.create("String", Position.createDummyPosition());
		final IdentifierToken integer = IdentifierToken.create("Integer", Position.createDummyPosition());
		final Name typeNameString = UnqualifiedName.create(string);
		final Name typeNameInteger = UnqualifiedName.create(integer);
		final BaseType stringType = BaseType.create(typeNameString, string, string);
		final BaseType integerType = BaseType.create(typeNameInteger, integer, integer);
		final SquareBracketOpenToken sbo = SquareBracketOpenToken.create(Position.createDummyPosition());
		final MapType mapType = MapType.create(sbo, stringType, integerType);
		final String expected = "[String->Integer]";
		
		assertEquals(expected, mapType.toString());
	}
}
