package de.fhdw.wtf.dsl.scanner.test;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.WhitespaceToken;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;

/**
 * Prüft mehrzeilige Scanner-Eingaben, sowie die bestimmten Positionen der Tokens und die Tabulatorlogik.
 */
public class ScannerMultilineTest {
	
	@Test
	public void lineIsCountedUp() {
		
		final VerboseTokenStream output = this.scanOutput("Zeile1\n" + "Zeile2\n" + "Zeile3", 2);
		
		assertEquals(IdentifierToken.create("Zeile1", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create("\n", Position.create("", 0 + 1, 6 + 1, 6)), output.next());
		assertEquals(IdentifierToken.create("Zeile2", Position.create("", 1 + 1, 0 + 1, 7)), output.next());
		assertEquals(WhitespaceToken.create("\n", Position.create("", 1 + 1, 6 + 1, 13)), output.next());
		assertEquals(IdentifierToken.create("Zeile3", Position.create("", 2 + 1, 0 + 1, 14)), output.next());
		assertEquals(EndToken.create(Position.create("", 2 + 1, 6 + 1, 20)), output.next());
	}
	
	@Test
	public void lineIsCountedUpWithCarriageReturn() {
		final VerboseTokenStream output = this.scanOutput("Zeile1\r\n" + "Zeile2\r\n" + "Zeile3", 2);
		
		assertEquals(IdentifierToken.create("Zeile1", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create("\r\n", Position.create("", 0 + 1, 6 + 1, 6)), output.next());
		assertEquals(IdentifierToken.create("Zeile2", Position.create("", 1 + 1, 0 + 1, 8)), output.next());
		assertEquals(WhitespaceToken.create("\r\n", Position.create("", 1 + 1, 6 + 1, 14)), output.next());
		assertEquals(IdentifierToken.create("Zeile3", Position.create("", 2 + 1, 0 + 1, 16)), output.next());
		assertEquals(EndToken.create(Position.create("", 2 + 1, 6 + 1, 22)), output.next());
	}
	
	@Test
	public void tabulatorCalculationWorks() {
		final VerboseTokenStream output = this.scanOutput("ab\tcd\n" + "a\tbcd\n" + "\tabcd\n" + "abcd\tef", 2);
		
		assertEquals(IdentifierToken.create("ab", Position.create("", 0 + 1, 0 + 1, 0)), output.next());
		assertEquals(WhitespaceToken.create("\t", Position.create("", 0 + 1, 2 + 1, 2)), output.next());
		assertEquals(IdentifierToken.create("cd", Position.create("", 0 + 1, 4 + 1, 3)), output.next());
		assertEquals(WhitespaceToken.create("\n", Position.create("", 0 + 1, 6 + 1, 5)), output.next());
		
		assertEquals(IdentifierToken.create("a", Position.create("", 1 + 1, 0 + 1, 6)), output.next());
		assertEquals(WhitespaceToken.create("\t", Position.create("", 1 + 1, 1 + 1, 7)), output.next());
		assertEquals(IdentifierToken.create("bcd", Position.create("", 1 + 1, 2 + 1, 8)), output.next());
		assertEquals(WhitespaceToken.create("\n\t", Position.create("", 1 + 1, 5 + 1, 11)), output.next());
		
		assertEquals(IdentifierToken.create("abcd", Position.create("", 2 + 1, 2 + 1, 13)), output.next());
		assertEquals(WhitespaceToken.create("\n", Position.create("", 2 + 1, 6 + 1, 17)), output.next());
		
		assertEquals(IdentifierToken.create("abcd", Position.create("", 3 + 1, 0 + 1, 18)), output.next());
		assertEquals(WhitespaceToken.create("\t", Position.create("", 3 + 1, 4 + 1, 22)), output.next());
		assertEquals(IdentifierToken.create("ef", Position.create("", 3 + 1, 6 + 1, 23)), output.next());
		
		assertEquals(EndToken.create(Position.create("", 3 + 1, 8 + 1, 25)), output.next());
	}
	
	private VerboseTokenStream scanOutput(final String s, final int tabWidth) {
		
		final SimpleScannerInput input = new SimpleScannerInput(s);
		final VerboseTokenStream output = new VerboseTokenStream(SimpleTokenStream.create());
		final Scanner scanner = ModelDslScanner.create(tabWidth);
		scanner.scan(input, output);
		return output;
	}
	
}
