package de.fhdw.wtf.parser.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Vector;

import org.junit.Before;
import org.junit.Test;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.ast.AttributeModifier;
import de.fhdw.wtf.common.ast.AttributeModifierFindable;
import de.fhdw.wtf.common.ast.AttributeModifierMutable;
import de.fhdw.wtf.common.ast.AttributeModifierPrior;
import de.fhdw.wtf.common.ast.AttributeModifierSymmetric;
import de.fhdw.wtf.common.ast.AttributeModifierTransient;
import de.fhdw.wtf.common.ast.Constructor;
import de.fhdw.wtf.common.ast.Group;
import de.fhdw.wtf.common.ast.GroupElement;
import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.Operation;
import de.fhdw.wtf.common.ast.UnqualifiedName;
import de.fhdw.wtf.common.ast.type.BaseType;
import de.fhdw.wtf.common.ast.type.ByNameState;
import de.fhdw.wtf.common.ast.type.ClassModifier;
import de.fhdw.wtf.common.ast.type.ClassType;
import de.fhdw.wtf.common.ast.type.ListType;
import de.fhdw.wtf.common.ast.type.MapType;
import de.fhdw.wtf.common.ast.type.ProductElementType;
import de.fhdw.wtf.common.ast.type.ProductType;
import de.fhdw.wtf.common.ast.type.RegularClassType;
import de.fhdw.wtf.common.ast.type.SumType;
import de.fhdw.wtf.common.ast.type.Type;
import de.fhdw.wtf.common.ast.type.TypeProxy;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.stream.SimpleTokenStream;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.EndToken;
import de.fhdw.wtf.common.token.IdentifierToken;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.keywords.ClassToken;
import de.fhdw.wtf.common.token.keywords.FindableToken;
import de.fhdw.wtf.common.token.keywords.GroupToken;
import de.fhdw.wtf.common.token.keywords.MutableToken;
import de.fhdw.wtf.common.token.keywords.PriorToken;
import de.fhdw.wtf.common.token.keywords.SymmetricToken;
import de.fhdw.wtf.common.token.keywords.TransientToken;
import de.fhdw.wtf.common.token.symbols.ArrowToken;
import de.fhdw.wtf.common.token.symbols.AsteriskToken;
import de.fhdw.wtf.common.token.symbols.BracketCloseToken;
import de.fhdw.wtf.common.token.symbols.BracketOpenToken;
import de.fhdw.wtf.common.token.symbols.ColonToken;
import de.fhdw.wtf.common.token.symbols.CommaToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.CurlyBracketOpenToken;
import de.fhdw.wtf.common.token.symbols.EqualToken;
import de.fhdw.wtf.common.token.symbols.GreaterSymbolToken;
import de.fhdw.wtf.common.token.symbols.PlusSymbolToken;
import de.fhdw.wtf.common.token.symbols.SemicolonToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketCloseToken;
import de.fhdw.wtf.common.token.symbols.SquareBracketOpenToken;
import de.fhdw.wtf.parser.Parser;

public class TestAttributes {
	
	private TokenStream stream;
	private ColonToken colonToken;
	private GroupToken groupToken;
	private EqualToken equalToken;
	private SquareBracketOpenToken squareBracketOpenToken;
	private SquareBracketCloseToken squareBracketCloseToken;
	private SemicolonToken semicolonToken;
	private Parser parser;
	private ClassToken classToken;
	private CurlyBracketOpenToken curlyBracketOpenToken;
	private CurlyBracketCloseToken curlyBracketCloseToken;
	private EndToken endToken;
	private PlusSymbolToken plusSymbolToken;
	private GreaterSymbolToken greaterSymbol;
	private AsteriskToken asteriskToken;
	private IdentifierToken stringIdentifier;
	private IdentifierToken integerIdentifier;
	private BracketOpenToken bracketOpenToken;
	private BracketCloseToken bracketCloseToken;
	private CommaToken commaToken;
	private ArrowToken arrowToken;
	
	/**
	 * Creates the most important Tokens.
	 */
	@Before
	public void setUp() {
		this.stream = SimpleTokenStream.create();
		this.stringIdentifier = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		this.integerIdentifier = IdentifierToken.create("Integer", Position.create("", 0 + 1, 0 + 1, 0));
		this.colonToken = ColonToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.groupToken = GroupToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.equalToken = EqualToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.squareBracketOpenToken = SquareBracketOpenToken.create(Position.create("", 0, 0, 0));
		this.squareBracketCloseToken = SquareBracketCloseToken.create(Position.create("", 0, 0, 0));
		this.semicolonToken = SemicolonToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.parser = Parser.create(this.stream);
		this.classToken = ClassToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.curlyBracketOpenToken = CurlyBracketOpenToken.create(Position.create("", 0, 0, 0));
		this.curlyBracketCloseToken = CurlyBracketCloseToken.create(Position.create("", 0, 0, 0));
		this.endToken = EndToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.plusSymbolToken = PlusSymbolToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.greaterSymbol = GreaterSymbolToken.create(Position.create("", 0, 0, 0));
		this.asteriskToken = AsteriskToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.bracketOpenToken = BracketOpenToken.create(Position.create("", 1, 1, 0));
		this.bracketCloseToken = BracketCloseToken.create(Position.create("", 1, 1, 0));
		this.commaToken = CommaToken.create(Position.create("", 1, 1, 0));
		this.arrowToken = ArrowToken.create(Position.create("", 1, 1, 0));
	}
	
	/**
	 * Group:group=[Class:class={Attribute:String;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testOneAttribute() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group1 = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType class1 =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						new Vector<ClassType>());
		
		final ByNameState attributeTypeReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy attributeType = TypeProxy.create(this.stringIdentifier, attributeTypeReferenceState);
		final Attribute class1Attribute =
				Attribute.create("Attribute", attributeType, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(class1Attribute);
		groupVector.add(class1);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group1);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class= Superclass + {Attribute:String;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testClassWithSuperclassAndAttribute() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken superclassIdentifier =
				IdentifierToken.create("Superclass", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(superclassIdentifier);
		this.stream.add(this.plusSymbolToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		
		final Collection<GroupElement> groupVector = new Vector<>();
		final Collection<Attribute> classAttributes = new Vector<>();
		final Collection<Type> class1SuperTypes = new Vector<>();
		final Collection<AttributeModifier> attrModifiers = new Vector<>();
		
		final Name name = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(name, groupVector, groupIdentifier);
		final Name class1Name = name.addName(classIdentifier);
		
		final ClassType classModel =
				RegularClassType.create(
						class1Name,
						new Vector<ClassModifier>(),
						classAttributes,
						class1SuperTypes,
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final Name class2name = UnqualifiedName.create(superclassIdentifier);
		final ByNameState class2ReferenceState = ByNameState.create(class2name);
		final TypeProxy class2Reference = TypeProxy.create(superclassIdentifier, class2ReferenceState);
		class1SuperTypes.add(class2Reference);
		
		final Name attrTypeClass1Name = UnqualifiedName.create(this.stringIdentifier);
		final ByNameState attrClass1TypeReferenceState = ByNameState.create(attrTypeClass1Name);
		final TypeProxy attrClass1Type = TypeProxy.create(this.stringIdentifier, attrClass1TypeReferenceState);
		final Attribute class1Attribute =
				Attribute.create("Attribute", attrClass1Type, attrModifiers, attributeIdentifier);
		classAttributes.add(class1Attribute);
		groupVector.add(classModel);
		
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(expected, actual);
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:String;Attribute2:Integer;Attribute3 :String;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testSomeAttributes() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier1 =
				IdentifierToken.create("Attribute1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier2 =
				IdentifierToken.create("Attribute2", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier3 =
				IdentifierToken.create("Attribute3", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier1);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(attributeIdentifier2);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(attributeIdentifier3);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group1 = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType class1 =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeStringReferenceState =
				ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, attributeStringReferenceState);
		
		final ByNameState attributeIntegerReferenceState =
				ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, attributeIntegerReferenceState);
		
		final Attribute attribute1 =
				Attribute.create("Attribute1", stringType, new Vector<AttributeModifier>(), attributeIdentifier1);
		final Attribute attribute2 =
				Attribute.create("Attribute2", integerType, new Vector<AttributeModifier>(), attributeIdentifier2);
		final Attribute attribute3 =
				Attribute.create("Attribute3", stringType, new Vector<AttributeModifier>(), attributeIdentifier3);
		attributes.add(attribute1);
		attributes.add(attribute2);
		attributes.add(attribute3);
		groupVector.add(class1);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group1);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group1:group=[Class1:class={};];Group2:group=[Class2:class={Attribute: Class1;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithOwnClassType() throws Exception {
		final IdentifierToken group1Identifier = IdentifierToken.create("Group1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken group2Identifier = IdentifierToken.create("Group2", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken class1Identifier = IdentifierToken.create("Class1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken class2Identifier = IdentifierToken.create("Class2", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(group1Identifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(class1Identifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(group2Identifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(class2Identifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(class1Identifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> group1Vector = new Vector<>();
		final Vector<GroupElement> group2Vector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name group1Name = UnqualifiedName.create(group1Identifier);
		final Group group1 = Group.create(group1Name, group1Vector, group1Identifier);
		
		final Name class1Name = group1Name.addName(class1Identifier);
		final ClassType class1 =
				RegularClassType.create(
						class1Name,
						new Vector<ClassModifier>(),
						new Vector<Attribute>(),
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						class1Identifier,
						
						new Vector<ClassType>());
		
		final Name group2Name = UnqualifiedName.create(group2Identifier);
		final Group group2 = Group.create(group2Name, group2Vector, group2Identifier);
		
		final Name class2Name = group2Name.addName(class2Identifier);
		final ClassType class2 =
				RegularClassType.create(
						class2Name,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						class1Identifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeReferenceState = ByNameState.create(UnqualifiedName.create(class1Identifier));
		final TypeProxy classType = TypeProxy.create(class1Identifier, attributeReferenceState);
		
		final Attribute attribute1 =
				Attribute.create("Attribute", classType, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute1);
		group1Vector.add(class1);
		group2Vector.add(class2);
		final Model expected = Model.create(group1Identifier);
		expected.addGroup(group1);
		expected.addGroup(group2);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:String*;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithList() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy attributeType = TypeProxy.create(this.stringIdentifier, attributeReferenceState);
		final ListType listType = ListType.create(this.stringIdentifier, attributeType);
		
		final Attribute attribute =
				Attribute.create("Attribute", listType, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:String***;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithListInList() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy attributeType = TypeProxy.create(this.stringIdentifier, attributeReferenceState);
		final ListType list1 = ListType.create(this.stringIdentifier, attributeType);
		final ListType list2 = ListType.create(this.stringIdentifier, list1);
		final ListType list3 = ListType.create(this.stringIdentifier, list2);
		
		final Attribute attribute =
				Attribute.create("Attribute", list3, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:();};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithEmptyProduct() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		
		final Attribute attribute =
				Attribute.create("Attribute", product, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:(name:String,count:Integer)*;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithListOnProduct() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final ListType list = ListType.create(this.bracketOpenToken, product);
		
		final Attribute attribute =
				Attribute.create("Attribute", list, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:(name:String,count:Integer)**;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMoreListsOnProduct() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(IdentifierToken.create("name", Position.create("", 1, 1, 0)));
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(IdentifierToken.create("count", Position.create("", 1, 1, 0)));
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final ListType list = ListType.create(this.bracketOpenToken, product);
		final ListType list2 = ListType.create(this.bracketOpenToken, list);
		
		final Attribute attribute =
				Attribute.create("Attribute", list2, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:{True, False}**;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMoreListsOnSum() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken trueIdentifier = IdentifierToken.create("true", Position.create("", 1, 1, 0));
		final IdentifierToken falseIdentifier = IdentifierToken.create("false", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(trueIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(falseIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState trueReferenceState = ByNameState.create(UnqualifiedName.create(trueIdentifier));
		final TypeProxy trueType = TypeProxy.create(trueIdentifier, trueReferenceState);
		final ByNameState falseReferenceState = ByNameState.create(UnqualifiedName.create(falseIdentifier));
		final TypeProxy falseType = TypeProxy.create(falseIdentifier, falseReferenceState);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.getElements().add(trueType);
		sum.getElements().add(falseType);
		
		final ListType list = ListType.create(this.bracketOpenToken, sum);
		final ListType list2 = ListType.create(this.bracketOpenToken, list);
		
		final Attribute attribute =
				Attribute.create("Attribute", list2, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[Integer->String];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeMap() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final MapType map = MapType.create(this.integerIdentifier, integerType, stringType);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[Integer*->String];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapAndListAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ListType list = ListType.create(this.integerIdentifier, integerType);
		
		final MapType map = MapType.create(this.integerIdentifier, list, stringType);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[Integer->String]*;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithListOfMap() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final MapType map = MapType.create(this.integerIdentifier, integerType, stringType);
		
		final ListType list = ListType.create(this.bracketOpenToken, map);
		
		final Attribute attribute =
				Attribute.create("Attribute", list, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[Integer->String]*;;];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithListOfMapButMissingBracketClose() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
		} catch (final NoValidTokenStreamException e) {
			// Erwartet
			return;
		}
		fail();
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[String->(name:String,count:Integer)] ;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapOnProduct() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.arrowToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final MapType map = MapType.create(this.stringIdentifier, stringType, product);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[(name:String,count:Integer)->Integer* ];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapOnProductAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final MapType map = MapType.create(this.bracketOpenToken, product, integerType);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:(name:.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapOnBeginningOfProduct() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(3, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[()->Integer];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapOnEmptyProductAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		
		final MapType map = MapType.create(this.bracketOpenToken, product, integerType);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[(name:String,count:Integer)->Integer ]*;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithListOfMapAndProductAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final MapType map = MapType.create(this.bracketOpenToken, product, integerType);
		final ListType list = ListType.create(this.bracketOpenToken, map);
		
		final Attribute attribute =
				Attribute.create("Attribute", list, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[(name:String,count:Integer)->Integer ]**;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithListOfListOfMapAndProductAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final MapType map = MapType.create(this.bracketOpenToken, product, integerType);
		final ListType list = ListType.create(this.bracketOpenToken, map);
		final ListType outerList = ListType.create(this.bracketOpenToken, list);
		
		final Attribute attribute =
				Attribute.create("Attribute", outerList, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[ [(name:String,count:Integer)->Integer]->String];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapAndMapAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final MapType mapKey = MapType.create(this.bracketOpenToken, product, integerType);
		final MapType map = MapType.create(this.bracketOpenToken, mapKey, stringType);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:[(name:String,count:Integer)* -> String];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithMapAndListOfProductAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken countIdentifier = IdentifierToken.create("count", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(countIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState stringReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy stringType = TypeProxy.create(this.stringIdentifier, stringReferenceState);
		final ByNameState integerReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy integerType = TypeProxy.create(this.integerIdentifier, integerReferenceState);
		
		final ProductElementType nameElement = ProductElementType.create("name", stringType, nameIdentifier);
		final ProductElementType countElement = ProductElementType.create("count", integerType, countIdentifier);
		final ProductType product = ProductType.create(this.bracketOpenToken);
		product.addElement(nameElement);
		product.addElement(countElement);
		
		final ListType productList = ListType.create(this.bracketOpenToken, product);
		
		final MapType map = MapType.create(this.bracketOpenToken, productList, stringType);
		
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:Integer prior;};.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithModifier() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(PriorToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy type = TypeProxy.create(this.stringIdentifier, attributeReferenceState);
		
		final Vector<AttributeModifier> modifiers = new Vector<>();
		modifiers.add(AttributeModifierPrior.create(PriorToken.create(Position.create("", 0, 0, 0))));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group:group=[Class:class={Attribute:Integer prior transient findable mutable;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithAllModifiers() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(PriorToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(TransientToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(FindableToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(MutableToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeReferenceState = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy type = TypeProxy.create(this.stringIdentifier, attributeReferenceState);
		
		final Vector<AttributeModifier> modifiers = new Vector<>();
		modifiers.add(AttributeModifierPrior.create(PriorToken.create(Position.create("", 0, 0, 0))));
		modifiers.add(AttributeModifierTransient.create(TransientToken.create(Position.create("", 0 + 1, 0 + 1, 0))));
		modifiers.add(AttributeModifierFindable.create(FindableToken.create(Position.create("", 0 + 1, 0 + 1, 0))));
		modifiers.add(AttributeModifierMutable.create(MutableToken.create(Position.create("", 0 + 1, 0 + 1, 0))));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group1:group=[Class1:class={name:String prior prior;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testSimpleClassInGroupWithPriorModifiersComplete() {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(PriorToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(PriorToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		try {
			this.parser.parse();
			fail();
		} catch (final NoValidTokenStreamException e) {
			assertEquals(1, this.parser.getExceptions().size());
		}
	}
	
	/**
	 * Group1:group=[Class1:class={name:Group1>Class1;};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testClassWithAttributeQualifiedType() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(groupIdentifier);
		this.stream.add(this.greaterSymbol);
		this.stream.add(classIdentifier);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 0 + 1, 0 + 1, 0)));
		
		final Model model = Model.create(groupIdentifier);
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		model.addGroup(group);
		
		final Name class1Name = groupName.addName(classIdentifier);
		final Collection<Attribute> classAttributes = new Vector<>();
		final ClassType class1Model =
				RegularClassType.create(
						class1Name,
						new Vector<ClassModifier>(),
						classAttributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		groupVector.add(class1Model);
		
		Name attrTypeClass1Name = UnqualifiedName.create(groupIdentifier);
		attrTypeClass1Name = attrTypeClass1Name.addName(classIdentifier);
		final ByNameState attrClassTypeReferenceState = ByNameState.create(attrTypeClass1Name);
		final TypeProxy classType = TypeProxy.create(groupIdentifier, attrClassTypeReferenceState);
		final Collection<AttributeModifier> attrModifiers = new Vector<>();
		final Attribute classAttribute = Attribute.create("Attribute", classType, attrModifiers, groupIdentifier);
		classAttributes.add(classAttribute);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(model, actual);
	}
	
	/**
	 * Group:group=[Class:class={Attribute1:Integer symmetric (Attribute2);};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testAttributeWithSymmertricModifier() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier1 =
				IdentifierToken.create("Attribute1", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier2 =
				IdentifierToken.create("Attribute2", Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier1);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		final SymmetricToken symmetrictoken = SymmetricToken.create(Position.create("", 0 + 1, 0 + 1, 0));
		this.stream.add(symmetrictoken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(attributeIdentifier2);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.endToken);
		
		final Vector<GroupElement> groupVector = new Vector<>();
		final Vector<Attribute> attributes = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		
		final Name className = groupName.addName(classIdentifier);
		final ClassType classModel =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		
		final ByNameState attributeReferenceState = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy type = TypeProxy.create(this.integerIdentifier, attributeReferenceState);
		
		final Vector<AttributeModifier> modifiers = new Vector<>();
		modifiers.add(AttributeModifierSymmetric.create(symmetrictoken, attributeIdentifier2, this.semicolonToken));
		final Attribute attribute = Attribute.create("Attribute1", type, modifiers, attributeIdentifier1);
		attributes.add(attribute);
		groupVector.add(classModel);
		final Model expected = Model.create(groupIdentifier);
		expected.addGroup(group);
		
		final Model actual = this.parser.parse();
		assertEquals(expected, actual);
		assertEquals(0, this.parser.getExceptions().size());
	}
	
	/**
	 * Group1:group=[Class1:class={attribute:[ [Integer->Integer]->String];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testMapAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.arrowToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 1, 1, 0)));
		
		final Model model = Model.create(groupIdentifier);
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		model.addGroup(group);
		
		final Name className = groupName.addName(classIdentifier);
		final Collection<Attribute> attributes = new Vector<>();
		final ClassType clazz =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		groupVector.add(clazz);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(groupIdentifier, stringByName);
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(groupIdentifier, integerByName);
		
		final MapType map1 = MapType.create(this.integerIdentifier, typeProxyInteger, typeProxyInteger);
		final MapType map2 = MapType.create(this.integerIdentifier, map1, typeProxyString);
		
		final Attribute attribute =
				Attribute.create("Attribute", map2, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(model, actual);
	}
	
	/**
	 * Group1:group=[Class1:class={attribute:[{Integer, String}->{Integer, String}];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testSumAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 1, 1, 0)));
		
		final Model model = Model.create(groupIdentifier);
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		model.addGroup(group);
		
		final Name className = groupName.addName(classIdentifier);
		final Collection<Attribute> attributes = new Vector<>();
		final ClassType clazz =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		groupVector.add(clazz);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(groupIdentifier, stringByName);
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(groupIdentifier, integerByName);
		
		final SumType sum = SumType.create(this.integerIdentifier);
		sum.add(typeProxyInteger);
		sum.add(typeProxyString);
		
		final MapType map = MapType.create(this.curlyBracketOpenToken, sum, sum);
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(model, actual);
	}
	
	/**
	 * Group1:group=[Class1:class={attribute:[(name:String, count:Integer)->{True, False}];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testProductAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken nameIdentifier = IdentifierToken.create("name", Position.create("", 1, 1, 0));
		final IdentifierToken productIdentifier = IdentifierToken.create("product", Position.create("", 1, 1, 0));
		final IdentifierToken trueIdentifier = IdentifierToken.create("true", Position.create("", 1, 1, 0));
		final IdentifierToken falseIdentifier = IdentifierToken.create("false", Position.create("", 1, 1, 0));
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.bracketOpenToken);
		this.stream.add(nameIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(productIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.integerIdentifier);
		this.stream.add(this.bracketCloseToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(trueIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(falseIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 1, 1, 0)));
		
		final Model model = Model.create(groupIdentifier);
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		model.addGroup(group);
		
		final Name className = groupName.addName(classIdentifier);
		final Collection<Attribute> attributes = new Vector<>();
		final ClassType clazz =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		groupVector.add(clazz);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(groupIdentifier, stringByName);
		
		final ByNameState integerByName = ByNameState.create(UnqualifiedName.create(this.integerIdentifier));
		final TypeProxy typeProxyInteger = TypeProxy.create(groupIdentifier, integerByName);
		
		final ByNameState trueByName = ByNameState.create(UnqualifiedName.create(trueIdentifier));
		final TypeProxy typeProxyTrue = TypeProxy.create(groupIdentifier, trueByName);
		
		final ByNameState falseByName = ByNameState.create(UnqualifiedName.create(falseIdentifier));
		final TypeProxy typeProxyFalse = TypeProxy.create(groupIdentifier, falseByName);
		
		final ProductType product = ProductType.create(this.bracketOpenToken);
		final ProductElementType element1 = ProductElementType.create("name", typeProxyString, nameIdentifier);
		final ProductElementType element2 = ProductElementType.create("product", typeProxyInteger, productIdentifier);
		product.addElement(element1);
		product.addElement(element2);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(typeProxyTrue);
		sum.add(typeProxyFalse);
		
		final MapType map = MapType.create(this.curlyBracketOpenToken, product, sum);
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(model, actual);
	}
	
	/**
	 * Group1:group=[Class1:class={attribute:[String*->{True, False}];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testListAsKey() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken trueIdentifier = IdentifierToken.create("true", Position.create("", 1, 1, 0));
		final IdentifierToken falseIdentifier = IdentifierToken.create("false", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(trueIdentifier);
		this.stream.add(this.commaToken);
		this.stream.add(falseIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 1, 1, 0)));
		
		final Model model = Model.create(groupIdentifier);
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		model.addGroup(group);
		
		final Name className = groupName.addName(classIdentifier);
		final Collection<Attribute> attributes = new Vector<>();
		final ClassType clazz =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		groupVector.add(clazz);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(groupIdentifier, stringByName);
		
		final ByNameState trueByName = ByNameState.create(UnqualifiedName.create(trueIdentifier));
		final TypeProxy typeProxyTrue = TypeProxy.create(groupIdentifier, trueByName);
		
		final ByNameState falseByName = ByNameState.create(UnqualifiedName.create(falseIdentifier));
		final TypeProxy typeProxyFalse = TypeProxy.create(groupIdentifier, falseByName);
		
		final ListType list = ListType.create(this.stringIdentifier, typeProxyString);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(typeProxyTrue);
		sum.add(typeProxyFalse);
		
		final MapType map = MapType.create(this.curlyBracketOpenToken, list, sum);
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(model, actual);
	}
	
	/**
	 * Group1:group=[Class1:class={attribute:[String ->{{True}*, False}];};];.
	 * 
	 * @throws Exception
	 *             : {@link NoValidTokenStreamException}
	 */
	@Test
	public void testSumWithListOfSum() throws Exception {
		final IdentifierToken groupIdentifier = IdentifierToken.create("Group", Position.create("", 1, 1, 0));
		final IdentifierToken classIdentifier = IdentifierToken.create("Class", Position.create("", 1, 1, 0));
		final IdentifierToken attributeIdentifier = IdentifierToken.create("Attribute", Position.create("", 1, 1, 0));
		final IdentifierToken trueIdentifier = IdentifierToken.create("true", Position.create("", 1, 1, 0));
		final IdentifierToken falseIdentifier = IdentifierToken.create("false", Position.create("", 1, 1, 0));
		
		this.stream.add(groupIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.groupToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(classIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.classToken);
		this.stream.add(this.equalToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(attributeIdentifier);
		this.stream.add(this.colonToken);
		this.stream.add(this.squareBracketOpenToken);
		this.stream.add(this.stringIdentifier);
		this.stream.add(this.arrowToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(this.curlyBracketOpenToken);
		this.stream.add(trueIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.asteriskToken);
		this.stream.add(this.commaToken);
		this.stream.add(falseIdentifier);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.curlyBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(this.squareBracketCloseToken);
		this.stream.add(this.semicolonToken);
		this.stream.add(EndToken.create(Position.create("", 1, 1, 0)));
		
		final Model model = Model.create(groupIdentifier);
		final Vector<GroupElement> groupVector = new Vector<>();
		
		final Name groupName = UnqualifiedName.create(groupIdentifier);
		final Group group = Group.create(groupName, groupVector, groupIdentifier);
		model.addGroup(group);
		
		final Name className = groupName.addName(classIdentifier);
		final Collection<Attribute> attributes = new Vector<>();
		final ClassType clazz =
				RegularClassType.create(
						className,
						new Vector<ClassModifier>(),
						attributes,
						new Vector<Type>(),
						new Vector<Operation>(),
						new Vector<Constructor>(),
						classIdentifier,
						
						new Vector<ClassType>());
		groupVector.add(clazz);
		
		final ByNameState stringByName = ByNameState.create(UnqualifiedName.create(this.stringIdentifier));
		final TypeProxy typeProxyString = TypeProxy.create(groupIdentifier, stringByName);
		
		final ByNameState trueByName = ByNameState.create(UnqualifiedName.create(trueIdentifier));
		final TypeProxy typeProxyTrue = TypeProxy.create(groupIdentifier, trueByName);
		
		final ByNameState falseByName = ByNameState.create(UnqualifiedName.create(falseIdentifier));
		final TypeProxy typeProxyFalse = TypeProxy.create(groupIdentifier, falseByName);
		
		final SumType sumInner = SumType.create(this.curlyBracketOpenToken);
		sumInner.add(typeProxyTrue);
		final ListType list = ListType.create(this.curlyBracketOpenToken, sumInner);
		
		final SumType sum = SumType.create(this.curlyBracketOpenToken);
		sum.add(list);
		sum.add(typeProxyFalse);
		
		final MapType map = MapType.create(this.curlyBracketOpenToken, typeProxyString, sum);
		final Attribute attribute =
				Attribute.create("Attribute", map, new Vector<AttributeModifier>(), attributeIdentifier);
		attributes.add(attribute);
		
		final Model actual = this.parser.parse();
		assertEquals(0, this.parser.getExceptions().size());
		assertEquals(model, actual);
	}
	
	/**
	 * Attribute:String prior;.
	 */
	@Test
	public void testAttributeToString() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken prior = IdentifierToken.create("prior", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final Collection<AttributeModifier> modifiers = new ArrayList<AttributeModifier>();
		modifiers.add(AttributeModifierPrior.create(prior));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		
		assertEquals("Attribute:String prior;", attribute.toString());
	}
	
	/**
	 * Attribute:String findable;.
	 */
	@Test
	public void testAttributeIsFindable() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken findable = IdentifierToken.create("findable", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final Collection<AttributeModifier> modifiers = new ArrayList<>();
		modifiers.add(AttributeModifierFindable.create(findable));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		
		assertTrue(attribute.isFindable());
		assertFalse(attribute.isPrior());
		assertFalse(attribute.isTransient());
		assertFalse(attribute.isMutable());
		assertFalse(attribute.isSymmetric());
	}
	
	/**
	 * Attribute:String prior;.
	 */
	@Test
	public void testAttributeIsPrior() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken prior = IdentifierToken.create("prior", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final Collection<AttributeModifier> modifiers = new ArrayList<>();
		modifiers.add(AttributeModifierPrior.create(prior));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		
		assertFalse(attribute.isFindable());
		assertTrue(attribute.isPrior());
		assertFalse(attribute.isTransient());
		assertFalse(attribute.isMutable());
		assertFalse(attribute.isSymmetric());
	}
	
	/**
	 * Attribute:String transient;.
	 */
	@Test
	public void testAttributeIsTransient() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken trans = IdentifierToken.create("transient", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final Collection<AttributeModifier> modifiers = new ArrayList<>();
		modifiers.add(AttributeModifierTransient.create(trans));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		
		assertFalse(attribute.isFindable());
		assertFalse(attribute.isPrior());
		assertTrue(attribute.isTransient());
		assertFalse(attribute.isMutable());
		assertFalse(attribute.isSymmetric());
	}
	
	/**
	 * Attribute:String mutable;.
	 */
	@Test
	public void testAttributeIsMutable() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken mutable = IdentifierToken.create("mutable", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final Collection<AttributeModifier> modifiers = new ArrayList<>();
		modifiers.add(AttributeModifierMutable.create(mutable));
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		
		assertFalse(attribute.isFindable());
		assertFalse(attribute.isPrior());
		assertFalse(attribute.isTransient());
		assertTrue(attribute.isMutable());
		assertFalse(attribute.isSymmetric());
	}
	
	/**
	 * Attribute:String mutable;.
	 */
	@Test
	public void testAttributeHashCoce() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final IdentifierToken mutable = IdentifierToken.create("mutable", Position.createDummyPosition());
		final Collection<AttributeModifier> modifiers1 = new ArrayList<>();
		modifiers1.add(AttributeModifierMutable.create(mutable));
		final Attribute attribute1 = Attribute.create("Attribute", type, modifiers1, attributeIdentifier);
		final Collection<AttributeModifier> modifiers2 = new ArrayList<>();
		modifiers2.add(AttributeModifierMutable.create(mutable));
		final Attribute attribute2 = Attribute.create("Attribute", type, modifiers2, attributeIdentifier);
		
		assertEquals(attribute1, attribute2);
		assertEquals(attribute1.hashCode(), attribute2.hashCode());
	}
	
	/**
	 * Attribute:Integer symmetric (Attribute2);.
	 */
	@Test
	public void testAttributeIsSymmetric() {
		final IdentifierToken string = IdentifierToken.create("String", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken attributeIdentifier =
				IdentifierToken.create("Attribute", Position.create("", 0 + 1, 0 + 1, 0));
		final IdentifierToken symmetric = IdentifierToken.create("symmetric", Position.create("", 0 + 1, 0 + 1, 0));
		final UnqualifiedName name = UnqualifiedName.create(string);
		final BaseType type = BaseType.create(name, string, string);
		final Collection<AttributeModifier> modifiers = new ArrayList<>();
		final AttributeModifierSymmetric modifier = AttributeModifierSymmetric.create(symmetric, symmetric, symmetric);
		modifiers.add(modifier);
		final Attribute attribute = Attribute.create("Attribute", type, modifiers, attributeIdentifier);
		
		assertFalse(attribute.isFindable());
		assertFalse(attribute.isPrior());
		assertFalse(attribute.isTransient());
		assertFalse(attribute.isMutable());
		assertTrue(attribute.isSymmetric());
	}
}
