package de.fhdw.wtf.walker.tasks.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.exception.walker.CyclicPartDefinitionException;
import de.fhdw.wtf.common.exception.walker.InvalidOverloadingException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.GroupDependencyTask;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.CyclicInheritanceCheck;
import de.fhdw.wtf.walker.tasks.DoubleAttributenameCheck;
import de.fhdw.wtf.walker.tasks.DoubleGroupcomponentCheck;
import de.fhdw.wtf.walker.tasks.OverloadingCheck;
import de.fhdw.wtf.walker.tasks.PrototypesTask;
import de.fhdw.wtf.walker.tasks.SubtypesFillTask;
import de.fhdw.wtf.walker.tasks.TypeReferencer;
import de.fhdw.wtf.walker.walker.SimpleWalkerTask;

/**
 * Test {@link OverloadingCheck}.
 */
public class TestOverloadingCheck extends TestCase {
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];operation : [[(parameter:Integer) ->
	 * String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperations() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];"
						+ "operation : [[(parameter:Integer) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(7, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 75, 74), ex.getStartPosition());
			assertEquals(Position.create("", 1, 122, 121), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Klasse) -> String ]];operation : [[(parameter:Klasse) ->
	 * String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperations2() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Klasse) -> String ]];"
						+ "operation : [[(parameter:Klasse) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(7, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 74, 73), ex.getStartPosition());
			assertEquals(Position.create("", 1, 120, 119), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Klasse2) -> String ]];operation : [[(parameter:Klasse2) ->
	 * String ]];}; Klasse2:class={};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperations3() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Klasse2) -> String ]];"
						+ "operation : [[(parameter:Klasse2) -> String ]];}; Klasse2:class={};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(7, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 75, 74), ex.getStartPosition());
			assertEquals(Position.create("", 1, 122, 121), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:{Integer,String}) -> String ]];operation :
	 * [[(parameter:{String,Integer}) -> String ]];}; Klasse2:class={};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperations4Sum() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:{Integer,String}) -> String ]];"
								+ "operation : [[(parameter:{String,Integer}) -> String ]];}; Klasse2:class={};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(7, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 84, 83), ex.getStartPosition());
			assertEquals(Position.create("", 1, 140, 139), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:(p1:Integer,p2:String)) -> String ]];operation :
	 * [[(parameter:(p1:Integer,p2:String)) -> String ]];}; Klasse2:class={};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperations5Product() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:(p1:Integer,p2:String)) -> String ]];"
								+ "operation : [[(parameter:(p1:Integer,p2:String)) -> String ]];}; Klasse2:class={};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(7, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 90, 89), ex.getStartPosition());
			assertEquals(Position.create("", 1, 152, 151), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[() -> String ]];operation : [[() -> String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperationsWithNoParameter() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[() -> String ]];operation : [[() -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(7, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 58, 57), ex.getStartPosition());
			assertEquals(Position.create("", 1, 88, 87), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(param1:Integer,param2:String) -> String ]];operation :
	 * [[(param1:String,param2:Integer) -> String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperationsButDifferentTypeSort() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(param1:Integer,param2:String) -> String ]];"
								+ "operation : [[(param1:String,param2:Integer) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * "Group:group=[Klasse:class={operation : [[(param:(p1:Integer,p2:String)) -> String ]];operation :
	 * [[(param:(p001:Integer,p002:String)) -> String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoDifferentProductsAsParamDifferentInProductElementNames() throws InterruptedException,
			ExecutionException, NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(param:(p1:Integer,p2:String)) -> String ]];"
								+ "operation : [[(param:(p001:Integer,p002:String)) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(param1:Integer,param2:String, param3:Integer) -> String ]];operation :
	 * [[(param1:String,param2:Integer,param3:Integer) -> String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperationsButDifferentTypeSort2() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(param1:Integer,param2:String, param3:Integer) -> String ]];"
								+ "operation : [[(param1:String,param2:Integer,param3:Integer) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];operation : [[(parameter:String) ->
	 * String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoOperationWithCorrectOverloading() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];"
						+ "operation : [[(parameter:String) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter2:String) -> String ]];operation :
	 * [[(parameter:String,parameter2:String) -> String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoOperationWithCorrectOverloading2() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter2:String) -> String ]];"
								+ "operation : [[(parameter:String,parameter2:String) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter2:String) -> String ]];operation :
	 * [[(parameter:Integer,parameter2:String,parameter3:Integer) -> String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoOperationWithCorrectOverloadingDifferentParameterCount() throws InterruptedException,
			ExecutionException, NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter2:String) -> String ]];"
								+ "operation : [[(parameter:Integer,parameter2:String,parameter3:Integer) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter2:String) -> String ]];operation :
	 * [[(parameter:String,parameter2:String) -> String ]];operation : [[(parameter:Integer,parameter2:Integer) ->
	 * String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testThreeOperationWithCorrectOverloading() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter2:String) -> String ]];"
								+ "operation : [[(parameter:String,parameter2:String) -> String ]];"
								+ "operation : [[(parameter:Integer,parameter2:Integer) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];operation2 : [[(parameter:Integer) ->
	 * String ]];}; ]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoDifferentOperationNames() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];"
						+ "operation2 : [[(parameter:Integer) -> String ]];}; ];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:{A}) -> String ]];operation : [[(parameter:{A,B,C}) -> String
	 * ]];}; A:class={};B:class=A+{};C:class=A+{};];"); .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testNormallyEqualSumIfStandardized() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:{A}) -> String ]];"
						+ "operation : [[(parameter:{A,B,C}) -> String ]];}; A:class={};B:class=A+{};C:class=A+{};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		referencer.addDependency(doubleGroupcomponent);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		doubleAttributenameCheck.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(6, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidOverloadingException) {
			final InvalidOverloadingException ex = (InvalidOverloadingException) exception;
			assertEquals(Position.create("", 1, 71, 70), ex.getStartPosition());
			assertEquals(Position.create("", 1, 118, 117), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];}; Klasse2:class={operation :
	 * [[(parameter:Integer) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualOperationsinDifferentClasses() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];}; "
						+ "Klasse2:class={operation : [[(parameter:Integer) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];};];Group2:group=[
	 * Klasse:class={operation : [[(parameter:Integer) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoEqualKlassesWithEqualOperationInDifferentGroups() throws InterruptedException,
			ExecutionException, NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Integer) -> String ]];};];"
						+ "Group2:group=[ Klasse:class={operation : [[(parameter:Integer) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Klasse) -> String ]];operation : [[(parameter: Group2>Klasse)
	 * -> String ]];}; Group2:group=[ Klasse:class={operation : [[(parameter:Klasse) -> String ]];};];]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 * @throws CyclicPartDefinitionException
	 */
	@Test
	public void testTwoOperationsWithDifferentTypeOfTwoGroups() throws InterruptedException, ExecutionException,
			NoValidTokenStreamException, CyclicDependencyException, CyclicPartDefinitionException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={operation : [[(parameter:Klasse) -> String ]];"
						+ "operation : [[(parameter: Group2>Klasse) -> String ]];}; "
						+ "Group2:group=[ Klasse:class={operation : [[(parameter:Klasse) -> String ]];};];];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		
		final SimpleWalkerTask doubleGroupcomponent = DoubleGroupcomponentCheck.create(model, taskmanager);
		final SimpleWalkerTask doubleAttributenameCheck = DoubleAttributenameCheck.create(model, taskmanager);
		final SimpleWalkerTask referencer = TypeReferencer.create(model, taskmanager);
		final SimpleWalkerTask prototypesTask = PrototypesTask.create(model, taskmanager);
		final SimpleWalkerTask cyclicInheritance = CyclicInheritanceCheck.create(model, taskmanager);
		final SimpleWalkerTask subtypesFillTask = SubtypesFillTask.create(model, taskmanager);
		final GroupDependencyTask doubleChecks = GroupDependencyTask.create(taskmanager);
		final SimpleWalkerTask overloadingCheck = OverloadingCheck.create(model, taskmanager);
		doubleChecks.addMembers(doubleGroupcomponent, doubleAttributenameCheck);
		referencer.addDependency(doubleChecks);
		cyclicInheritance.addDependency(referencer);
		subtypesFillTask.addDependency(cyclicInheritance);
		prototypesTask.addDependency(subtypesFillTask);
		overloadingCheck.addDependency(prototypesTask);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(8, okResult.size());
		assertEquals(0, failResult.size());
	}
}
