package de.fhdw.wtf.walker.tasks.test;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ExecutionException;

import junit.framework.TestCase;

import org.junit.Test;

import de.fhdw.wtf.common.ast.Model;
import de.fhdw.wtf.common.exception.parser.NoValidTokenStreamException;
import de.fhdw.wtf.common.exception.walker.CyclicDependencyException;
import de.fhdw.wtf.common.exception.walker.InvalidProductException;
import de.fhdw.wtf.common.stream.FilteredTokenStream;
import de.fhdw.wtf.common.stream.SimpleScannerInput;
import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.task.TaskExecutorFixed;
import de.fhdw.wtf.common.task.result.ExceptionalTaskResult;
import de.fhdw.wtf.common.task.result.OKTaskResult;
import de.fhdw.wtf.common.task.result.TaskResult;
import de.fhdw.wtf.common.task.result.visitor.TaskResultVisitor;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.Scanner;
import de.fhdw.wtf.dsl.scanner.modelScanner.ModelDslScanner;
import de.fhdw.wtf.parser.Parser;
import de.fhdw.wtf.walker.tasks.ProductCheck;

/**
 * Tests {@link ProductCheck}.
 */
public class TestProductCheck extends TestCase {
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter:String) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testSameNameTwice() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter:String) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidProductException) {
			final InvalidProductException ex = (InvalidProductException) exception;
			assertEquals(Position.create("", 1, 42, 41), ex.getStartPosition());
			assertEquals(Position.create("", 1, 79, 78), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={attr : (parameter:Integer,parameter:String);};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testSameNameTwice2() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={attr : (parameter:Integer,parameter:String);};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidProductException) {
			final InvalidProductException ex = (InvalidProductException) exception;
			assertEquals(Position.create("", 1, 35, 34), ex.getStartPosition());
			assertEquals(Position.create("", 1, 71, 70), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter:String,parameter:String) -> String ]];};];
	 * .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testSameNameThreeTimes() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,parameter:String,parameter:String) "
								+ "-> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidProductException) {
			final InvalidProductException ex = (InvalidProductException) exception;
			assertEquals(Position.create("", 1, 42, 41), ex.getStartPosition());
			assertEquals(Position.create("", 1, 96, 95), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={attr : (parameter:Integer,parameter:String,parameter:String);};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testSameNameThreeTimes2() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={attr : (parameter:Integer,parameter:String,parameter:String);};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(0, okResult.size());
		assertEquals(1, failResult.size());
		
		final Exception exception = failResult.iterator().next().getError();
		if (exception instanceof InvalidProductException) {
			final InvalidProductException ex = (InvalidProductException) exception;
			assertEquals(Position.create("", 1, 35, 34), ex.getStartPosition());
			assertEquals(Position.create("", 1, 88, 87), ex.getEndPos());
		} else {
			fail();
		}
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter1:Integer,parameter2:String) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testTwoDifferentNames() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter1:Integer,parameter2:String) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter1:Integer,parameter2:String) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testTwoDifferentNames2() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter1:Integer,parameter2:String) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,Parameter:String) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testCapitalisation() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,Parameter:String) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={operation : [[(parameter:Integer,paraMeter:String) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testCapitalisation2() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput(
						"Group:group=[Klasse:class={operation : [[(parameter:Integer,paraMeter:String) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={attr : (parameter:Integer,Parameter:String);};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testCapitalisation3() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={attr : (parameter:Integer,Parameter:String);};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={attr : (parameter:Integer,paraMeter:String);};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testCapitalisation4() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={attr : (parameter:Integer,paraMeter:String);};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<OKTaskResult>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<ExceptionalTaskResult>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
	
	/**
	 * Group:group=[Klasse:class={attr:(parameter:Integer,paraMeter:String);
	 * attr2:(parameter:Integer,paraMeter:String);operation : [[(parameter:Integer,paraMeter:String) -> String ]];};]; .
	 * 
	 * @throws InterruptedException
	 * @throws ExecutionException
	 * @throws NoValidTokenStreamException
	 * @throws CyclicDependencyException
	 */
	@Test
	public void testSameProductTwice() throws InterruptedException, ExecutionException, NoValidTokenStreamException,
			CyclicDependencyException {
		final SimpleScannerInput input =
				new SimpleScannerInput("Group:group=[Klasse:class={attr:(parameter:Integer,paraMeter:String);"
						+ "attr2:(parameter:Integer,paraMeter:String);"
						+ "operation : [[(parameter:Integer,paraMeter:String) -> String ]];};];");
		final TokenStream output = FilteredTokenStream.create();
		final Scanner scanner = ModelDslScanner.create();
		scanner.scan(input, output);
		final Parser parser = Parser.create(output);
		Model model = null;
		model = parser.parse();
		final TaskExecutorFixed taskmanager = TaskExecutorFixed.create();
		ProductCheck.create(model, taskmanager);
		taskmanager.startAllKnownTasks();
		
		final Collection<OKTaskResult> okResult = new ArrayList<>();
		final Collection<ExceptionalTaskResult> failResult = new ArrayList<>();
		final Collection<TaskResult> results = taskmanager.getResultsAndShutdown();
		for (final TaskResult current : results) {
			current.accept(new TaskResultVisitor() {
				
				@Override
				public void handleOkTaskResult(final OKTaskResult result) {
					okResult.add(result);
				}
				
				@Override
				public void handleExceptionalTaskResult(final ExceptionalTaskResult result) {
					failResult.add(result);
				}
			});
		}
		
		assertEquals(1, okResult.size());
		assertEquals(0, failResult.size());
	}
}
